unit UActiveRender;

interface

uses
  SysUtils, Classes, DBClient, DB, StdCtrls, dbWeb, ComCtrls;

type
  // Declaraes Forward
  TActiveRender = class;

  TActiveRenderField = class(TCollectionItem)
  private
   FFieldName: String;
  protected
     function GetDisplayName: string; override;
  published
    property FieldName: String read FFieldName write FFieldName;
  end;

  TActiveRenderFields = class(TCollection)
  private
    FActiveRender: TActiveRender;
  protected
    function GetItem(Index: Integer): TActiveRenderField;
    procedure SetItem(Index: Integer; Value: TActiveRenderField);
  public
    property ActiveRender: TActiveRender read FActiveRender;
    constructor Create(ActiveRender: TActiveRender);
    property Items[Index: Integer]: TActiveRenderField read GetItem write SetItem; Default;
  protected
    function  GetOwner: TPersistent; override;
  end;

  TActiveRender = class(TComponent)
  private
    FDataset: TDataset;
    FFileName: TFileName;
    FFieldSeparator: Char;
    FRenderFields: TActiveRenderFields;
    FOnAfterRender: TNotifyEvent;
  protected
    procedure ValidateProperties;
    procedure DoAfterRender; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RenderDataset;
  published
    property Dataset: TDataset read FDataset write FDataset;
    property FileName: TFileName read FFileName write FFileName;
    property FieldSeparator: Char read FFieldSeparator write FFieldSeparator;
    property RenderFields: TActiveRenderFields read FRenderFields write FRenderFields;
    property OnAfterRender: TNotifyEvent read FOnAfterRender write FOnAfterRender;
  end;


implementation

{ TActiveRender }

constructor TActiveRender.Create(AOwner: TComponent);
begin
  inherited;
  FRenderFields := TActiveRenderFields.Create(Self);
end;

destructor TActiveRender.Destroy;
begin
  if Assigned(FRenderFields) then
    FRenderFields.Free;
  inherited;
end;

procedure TActiveRender.DoAfterRender;
begin
  if Assigned(FOnAfterRender) then
    FOnAfterRender(Self);
end;


procedure TActiveRender.RenderDataset;
var
  i: integer;
  slLines: TStringList;
  slFields: TStringList;
begin
  // Executa validao das propriedades do renderizador
  ValidateProperties;

  // Adiciona todos os campos se nenhum foi adicionado
   if RenderFields.Count = 0 then
    for i := 0 to Dataset.Fields.Count - 1 do
     (RenderFields.Add as TActiveRenderField).FieldName := Dataset.Fields[i].FieldName;


  // Executa Renderizao do Dataset
   slLines := TStringList.Create;
   slFields := TStringList.Create;
   slFields.Delimiter := FieldSeparator;
   try
       Dataset.First;
       while not Dataset.Eof do begin
        slFields.Clear;
        for i := 0 to RenderFields.Count - 1 do
          slFields.Add(Dataset.FieldByName(RenderFields[i].FieldName).AsString);
        slLines.Add(slFields.DelimitedText);
        Dataset.Next;
       end;
       slLines.SaveToFile(FileName);
   finally
    slLines.Free;
    slFields.Free;
   end;

   // Dispara evento para processamento ps-renderizao pelo usurio
  DoAfterRender;
end;

procedure TActiveRender.ValidateProperties;
begin
   if not Assigned(FDataSet) then
     raise Exception.Create('DataSet em ' + ClassName + ' Faltando');
   if not FDataset.Active then
     raise Exception.Create('DataSet em ' + ClassName + ' no est ativo');
   if Dataset.Fields.Count = 0 then
     raise Exception.Create('Nenhum campo a renderizar.');
  if FileName = '' then
    raise Exception.Create('Nome do arquivo faltando');
  if FieldSeparator = #0 then
   raise Exception.Create('Caractere separador faltando');
end;

{ TActiveRenderField }

function TActiveRenderField.GetDisplayName: string;
begin
  Result := FieldName;
end;

{ TActiveRenderFields }

constructor TActiveRenderFields.Create(ActiveRender: TActiveRender);
begin
  FActiveRender := ActiveRender;
  inherited Create(TActiveRenderField);
end;

function TActiveRenderFields.GetItem(Index: Integer): TActiveRenderField;
begin
  Result := TActiveRenderField(Inherited GetItem(Index));
end;

function TActiveRenderFields.GetOwner: TPersistent;
begin
   Result := FActiveRender;
end;

procedure TActiveRenderFields.SetItem(Index: Integer; Value: TActiveRenderField);
begin
  inherited SetItem(Index,Value);
end;

end.
