unit UActiveRender;

interface

uses
  SysUtils, Classes, DBClient, DB, StdCtrls, dbWeb, ComCtrls;

type
  // Declaraes Forward
  TActiveRender = class;
  TActiveRenderTypes = (arText, arHTML);

  TActiveRenderField = class(TCollectionItem)
  private
   FFieldName: String;
  protected
     function GetDisplayName: string; override;
  published
    property FieldName: String read FFieldName write FFieldName;
  end;

  TActiveRenderFields = class(TCollection)
  private
    FActiveRender: TActiveRender;
  protected
    function GetItem(Index: Integer): TActiveRenderField;
    procedure SetItem(Index: Integer; Value: TActiveRenderField);
  public
    property ActiveRender: TActiveRender read FActiveRender;
    constructor Create(ActiveRender: TActiveRender);
    property Items[Index: Integer]: TActiveRenderField read GetItem write SetItem; Default;
  protected
    function  GetOwner: TPersistent; override;
  end;

  TActiveRender = class(TComponent)
  private
    FDataset: TDataset;
    FFileName: TFileName;
    FFieldSeparator: Char;
    FRenderFields: TActiveRenderFields;
    FOnAfterRender: TNotifyEvent;
    FRenderType: TActiveRenderTypes;
    FBorderWidth: Integer;
    FHTMLFileName: String;
  protected
    procedure ValidateProperties;
    procedure DoAfterRender; virtual;
    procedure RenderDatasetText;
    procedure RenderDatasetHTML;
    // Executa a seleo do algoritmo de renderizao
    procedure DoRenderDataset;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    // interface externa para executar renderizao
    procedure RenderDataset;
  published
    property Dataset: TDataset read FDataset write FDataset;
    property FileName: TFileName read FFileName write FFileName;
    property FieldSeparator: Char read FFieldSeparator write FFieldSeparator;
    property RenderFields: TActiveRenderFields read FRenderFields write FRenderFields;
    property OnAfterRender: TNotifyEvent read FOnAfterRender write FOnAfterRender;
    property RenderType: TActiveRenderTypes read FRenderType write FRenderType;
    property HTMLFileName: String read FHTMLFileName write FHTMLFileName;
    property BorderWidth: Integer read FBorderWidth write FBorderWidth;

  end;


implementation

{ TActiveRender }

constructor TActiveRender.Create(AOwner: TComponent);
begin
  inherited;
  FRenderFields := TActiveRenderFields.Create(Self);
end;

destructor TActiveRender.Destroy;
begin
  if Assigned(FRenderFields) then
    FRenderFields.Free;
  inherited;
end;

procedure TActiveRender.DoAfterRender;
begin
  if Assigned(FOnAfterRender) then
    FOnAfterRender(Self);
end;


procedure TActiveRender.DoRenderDataset;
begin
  case RenderType of
    arText: RenderDatasetText;
    arHTML: RenderDatasetHTML;
  end;
end;

procedure TActiveRender.RenderDataset;
var
  i: integer;
begin
  // Executa validao das propriedades do renderizador
  ValidateProperties;

  // Adiciona todos os campos se nenhum foi adicionado
   if RenderFields.Count = 0 then
    for i := 0 to Dataset.Fields.Count - 1 do
     (RenderFields.Add as TActiveRenderField).FieldName := Dataset.Fields[i].FieldName;

   // Executa Renderizao do Dataset
   DoRenderDataset;

   // Dispara evento para processamento ps-renderizao pelo usurio
  DoAfterRender;
end;

procedure TActiveRender.RenderDatasetHTML;
var
  TableProducer: TDataSetTableProducer;
  i: integer;
  sl: TStringList;
begin
    Dataset.First;
    TableProducer := TDataSetTableProducer.Create(nil);
    sl := TStringList.Create;
    try
      TableProducer.TableAttributes.Border := BorderWidth;
      for i := 0 to RenderFields.Count -1 do
        TableProducer.Columns.Add.FieldName := RenderFields[i].FieldName;
      TableProducer.DataSet := Dataset;
      sl.Text := TableProducer.Content;
      sl.SaveToFile(HTMLFileName);
    finally
     TableProducer.Free;
     sl.free;
    end;
end;

procedure TActiveRender.RenderDatasetText;
var
  slLines: TStringList;
  slFields: TStringList;
  i: integer;
begin
   slLines := TStringList.Create;
   slFields := TStringList.Create;
   slFields.Delimiter := FieldSeparator;
   try
       Dataset.First;
       while not Dataset.Eof do begin
        slFields.Clear;
        for i := 0 to RenderFields.Count - 1 do
          slFields.Add(Dataset.FieldByName(RenderFields[i].FieldName).AsString);
        slLines.Add(slFields.DelimitedText);
        Dataset.Next;
       end;
       slLines.SaveToFile(FileName);
   finally
    slLines.Free;
    slFields.Free;
   end;
end;

procedure TActiveRender.ValidateProperties;
begin
   if not Assigned(FDataSet) then
     raise Exception.Create('DataSet em ' + ClassName + ' Faltando');
   if not FDataset.Active then
     raise Exception.Create('DataSet em ' + ClassName + ' no est ativo');
   if Dataset.Fields.Count = 0 then
     raise Exception.Create('Nenhum campo a renderizar.');
  case RenderType of
    arText: begin
     if FileName = '' then
       raise Exception.Create('Nome do arquivo texto faltando');
     if FieldSeparator = #0 then
       raise Exception.Create('FieldSeparator Faltando');
    end;
    arHTML: begin
     if HTMLFileName = '' then
       raise Exception.Create('Nome do arquivo HTML faltando');
      if FBorderWidth < 1 then
        raise Exception.Create('Tamanho de borda invlido');
    end;
  end;
end;

{ TActiveRenderField }

function TActiveRenderField.GetDisplayName: string;
begin
  Result := FieldName;
end;

{ TActiveRenderFields }

constructor TActiveRenderFields.Create(ActiveRender: TActiveRender);
begin
  FActiveRender := ActiveRender;
  inherited Create(TActiveRenderField);
end;

function TActiveRenderFields.GetItem(Index: Integer): TActiveRenderField;
begin
  Result := TActiveRenderField(Inherited GetItem(Index));
end;

function TActiveRenderFields.GetOwner: TPersistent;
begin
   Result := FActiveRender;
end;

procedure TActiveRenderFields.SetItem(Index: Integer; Value: TActiveRenderField);
begin
  inherited SetItem(Index,Value);
end;

end.
