unit UActiveRenderEngines;

interface

uses
  SysUtils, Classes, DBClient, DB, StdCtrls, dbWeb, ComCtrls, UActiveRender;

type
  TActiveRenderEngineText = class(TActiveRenderEngine)
  private
    FFieldSeparator: Char;
    FFileName: TFileName;
  protected
   procedure DoValidateProperties; override;
   procedure DoRenderDataSet; override;
  published
    property FileName: TFileName read FFileName write FFileName;
    property FieldSeparator: Char read FFieldSeparator write FFieldSeparator;
  end;

  TActiveRenderEngineHTML = class(TActiveRenderEngine)
  private
    FFileName: TFileName;
    FBorderWidth: Integer;
  protected
    procedure DoValidateProperties; override;
    procedure DoRenderDataSet; override;
  published
    property FileName: TFileName read FFileName write FFileName;
    property BorderWidth: Integer read FBorderWidth write FBorderWidth default 1;
  end;

implementation

{ TActiveRenderEngineText }

procedure TActiveRenderEngineText.DoRenderDataSet;
var
  i: integer;
  slLines: TStringList;
  slFields: TStringList;
begin
   slLines := TStringList.Create;
   slFields := TStringList.Create;
   slFields.Delimiter := FieldSeparator;
   try
     with ActiveRender do begin
       Dataset.First;
       while not Dataset.Eof do begin
        slFields.Clear;
        for i := 0 to RenderFields.Count - 1 do
          slFields.Add(Dataset.FieldByName(RenderFields[i].FieldName).AsString);
        slLines.Add(slFields.DelimitedText);
        Dataset.Next;
       end;
       slLines.SaveToFile(FileName);
     end;
   finally
    slLines.Free;
    slFields.Free;
   end;
end;

procedure TActiveRenderEngineText.DoValidateProperties;
begin
  if FileName = '' then
    raise Exception.Create('Nome do arquivo faltando em ' + ClassName);
  if FieldSeparator = #0 then
    raise Exception.Create('Field Separator faltando em ' + ClassName);
end;

{ TActiveRenderEnginehtml }

procedure TActiveRenderEngineHTML.DoRenderDataSet;
var
  TableProducer: TDataSetTableProducer;
  i: integer;
  sl: TStringList;
begin
  with ActiveRender do begin
    Dataset.First;
    TableProducer := TDataSetTableProducer.Create(nil);
    sl := TStringList.Create;
    try
      TableProducer.TableAttributes.Border := BorderWidth;
      for i := 0 to RenderFields.Count -1 do
        TableProducer.Columns.Add.FieldName := RenderFields[i].FieldName;
      TableProducer.DataSet := Dataset;
      sl.Text := TableProducer.Content;
      sl.SaveToFile(FileName);
    finally
     TableProducer.Free;
     sl.free;
    end;
  end;
end;

procedure TActiveRenderEngineHTML.DoValidateProperties;
begin
  if FileName = '' then
    raise Exception.Create('Nome do arquivo faltando em ' + ClassName);
  if FBorderWidth < 1 then
    raise Exception.Create('Tamanho da borda invlido em ' + ClassName);
end;

end.
