unit UActiveRender;

interface

uses
  SysUtils, Classes, DB, StdCtrls, dbWeb, ComCtrls;

type
  // declaraes forward
  TActiveRender = class;
  TActiveRenderEngine = class;

  TActiveRenderField = class(TCollectionItem)
  private
   FFieldName: String;
  protected
     function GetDisplayName: string; override;
  published
    property FieldName: String read FFieldName write FFieldName;
  end;

  TActiveRenderFields = class(TCollection)
  private
    FActiveRender: TActiveRender;
  protected
    function GetItem(Index: Integer): TActiveRenderField;
    procedure SetItem(Index: Integer; Value: TActiveRenderField);
  public
    property ActiveRender: TActiveRender read FActiveRender;
    constructor Create(ActiveRender: TActiveRender);
    property Items[Index: Integer]: TActiveRenderField read GetItem write SetItem; Default;
  protected
    function  GetOwner: TPersistent; override;
  end;

  TActiveRender = class(TComponent)
  private
    FDataset: TDataset;
    FRenderEngine: TActiveRenderEngine;
    FRenderFields: TActiveRenderFields;
    procedure SetRenderEngine(const Value: TActiveRenderEngine);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RenderDataset;
  published
    property Dataset: TDataset read FDataset write FDataset;
    property RenderEngine: TActiveRenderEngine read FRenderEngine write SetRenderEngine;
    property RenderFields: TActiveRenderFields read FRenderFields write FRenderFields;
  end;

  TActiveRenderEngine = class(TComponent)
  private
    FOnAfterRender: TNotifyEvent;
    FActiveRender: TActiveRender;
  protected
    procedure DoAfterRender; virtual;
    procedure DoValidateProperties; virtual;
    procedure DoRenderDataSet; virtual;
  public
    property ActiveRender: TActiveRender read FActiveRender write FActiveRender;
    procedure RenderDataSet;
    procedure ValidateProperties;
  published
    property OnAfterRender: TNotifyEvent read FOnAfterRender write FOnAfterRender;
  end;

{$R UActiveRender.dcr}

implementation

{ TActiveRender }

constructor TActiveRender.Create(AOwner: TComponent);
begin
  inherited;
  FRenderFields := TActiveRenderFields.Create(Self);
end;

destructor TActiveRender.destroy;
begin
  if Assigned(FRenderFields) then
    FRenderFields.Free;
  inherited;
end;

procedure TActiveRender.RenderDataset;
begin
  if not Assigned(RenderEngine) then
    raise Exception.Create('RenderEngine Faltando');

  // Executa Renderizao do Dataset
  RenderEngine.RenderDataSet;
end;

procedure TActiveRender.SetRenderEngine(const Value: TActiveRenderEngine);
begin
  FRenderEngine := Value;
  if Assigned(FRenderEngine) then
    FRenderEngine.ActiveRender := self;
end;

{ TActiveRenderField }

constructor TActiveRenderFields.Create(ActiveRender: TActiveRender);
begin
  FActiveRender := ActiveRender;
  inherited Create(TActiveRenderField);
end;

function TActiveRenderFields.GetItem(Index: Integer): TActiveRenderField;
begin
  Result := TActiveRenderField(Inherited GetItem(Index));
end;

function TActiveRenderFields.GetOwner: TPersistent;
begin
   Result := FActiveRender;
end;

{ TActiveRenderField }

function TActiveRenderField.GetDisplayName: string;
begin
  Result := FieldName;
end;

{ TActiveRenderEngine }

procedure TActiveRenderEngine.DoAfterRender;
begin
  if Assigned(FOnAfterRender) then
    FOnAfterRender(Self);
end;

procedure TActiveRenderEngine.DoRenderDataSet;
begin
  raise Exception.Create('Mtodo #DoRenderDataSet no implementado na classe ' + ClassName);
end;

procedure TActiveRenderEngine.DoValidateProperties;
begin
  raise Exception.Create('Mtodo #DoValidateProperties no implementado na classe ' + ClassName);
end;

procedure TActiveRenderEngine.RenderDataSet;
var
  i: integer;
begin
  // Executa validao das propriedades do renderizador
  ValidateProperties;

  // Adiciona todos os campos se nenhum foi adicionado
  with ActiveRender Do
   if RenderFields.Count = 0 then
    for i := 0 to Dataset.Fields.Count - 1 do
     (RenderFields.Add as TActiveRenderField).FieldName := Dataset.Fields[i].FieldName;

  // Executa Renderizao do Dataset
  DoRenderDataSet;

  // Dispara evento para processamento ps-renderizao pelo usurio
  DoAfterRender;
end;

procedure TActiveRenderEngine.ValidateProperties;
begin
   if not Assigned(ActiveRender) then
    raise Exception.Create('ActiveRender Faltando.');
   if not Assigned(ActiveRender.DataSet) then
     raise Exception.Create('DataSet em ' + ActiveRender.Name + ' faltando');
   if not ActiveRender.Dataset.Active then
     raise Exception.Create('DataSet em ' + ActiveRender.Name + ' no est ativo');
   if ActiveRender.Dataset.Fields.Count = 0 then
     raise Exception.Create('Nenhum campo a renderizar.');
   DoValidateProperties;
end;

procedure TActiveRenderFields.SetItem(Index: Integer; Value: TActiveRenderField);
begin
  inherited SetItem(Index,Value);
end;

end.
